package client.gui;

import client.gui.GuiConnect.ServerInfo;
import client.gui.element.ActButton;
import client.gui.element.Label;
import client.gui.element.NavButton;
import client.gui.element.Textbox;
import client.gui.element.Textbox.Action;
import client.vars.CVarCategory;
import client.vars.Variable;
import common.color.TextColor;
import common.init.Config;
import common.network.IPlayer;

public class GuiServer extends Gui implements Textbox.Callback {
	public static final GuiServer INSTANCE = new GuiServer(null);
	
	private final ServerInfo server;

	private Textbox nameBox;
	private Textbox addrBox;
	private Textbox portBox;
	private Textbox userBox;
	private Textbox passBox;
	private Textbox accBox;
	private Label nameLabel;
	private Label addrLabel;
	private Label portLabel;
	private Label userLabel;
	private Label passLabel;
	private Label accLabel;
	
	public GuiServer(ServerInfo server) {
		this.server = server;
	}

    @Variable(name = "srv_last_address", category = CVarCategory.SYSTEM, min = 1, max = 128, display = "Letzte Server-Adresse")
	private String lastAddr = "";
	@Variable(name = "srv_last_port", category = CVarCategory.SYSTEM, min = 0, max = 65535, display = "Letzter Server-Port")
	private int lastPort = Config.PORT;
    @Variable(name = "srv_last_user", category = CVarCategory.SYSTEM, max = IPlayer.MAX_USER_LENGTH, display = "Letzter Server-Nutzer", validator = IPlayer.UserValidator.class)
	private String lastUser = "";
    @Variable(name = "srv_last_password", category = CVarCategory.SYSTEM, max = IPlayer.MAX_PASS_LENGTH, display = "Letztes Server-Passwort")
	private String lastPass = "";
    @Variable(name = "srv_last_access", category = CVarCategory.SYSTEM, max = IPlayer.MAX_PASS_LENGTH, display = "Letzter Server-Zugang")
	private String lastAcc = "";
	
	public void init(int width, int height) {
		if(this.server != null)
			this.nameBox = this.add(new Textbox(0, -50, 400, 24, 128, true, this, this.server.getName()));
		this.addrBox = this.add(new Textbox(0, 20, 400, 24, 128, true, this, this.server == null ? this.lastAddr : this.server.getAddress()));
		this.portBox = this.add(new Textbox(404, 20, 76, 24, 5, true, this, "" + (this.server == null ? this.lastPort : this.server.getPort())));
		this.userBox = this.add(new Textbox(0, 70, 220, 24, IPlayer.MAX_USER_LENGTH, true, this, IPlayer.VALID_USER, this.server == null ? this.lastUser : this.server.getUser()));
		this.passBox = this.add(new Textbox(0, 120, 480, 24, IPlayer.MAX_PASS_LENGTH, true, this, this.server == null ? this.lastPass : this.server.getPassword()));
		this.accBox = this.add(new Textbox(0, 170, 480, 24, IPlayer.MAX_PASS_LENGTH, true, this, this.server == null ? this.lastAcc : this.server.getAccess()));
		this.add(new ActButton(0, 220, 480, 24, new ActButton.Callback() {
			public void use(ActButton elem, ActButton.Mode action) {
				GuiServer.this.connect();
			}
		}, this.server == null ? "Verbinden" : (this.server.getName().isEmpty() ? "Hinzufügen" : "Übernehmen")));
		this.add(new NavButton(0, 250, 480, 24, this.server != null ? GuiConnect.INSTANCE : GuiMenu.INSTANCE, "Zurück"));
		if(this.server != null)
			this.nameLabel = this.add(new Label(0, -70, 410, 20, "Name", true));
		this.addrLabel = this.add(new Label(0, 0, 410, 20, "Adresse", true));
		this.portLabel = this.add(new Label(414, 0, 66, 20, "Port", true));
		this.userLabel = this.add(new Label(0, 50, 220, 20, "Nutzer", true));
		this.passLabel = this.add(new Label(0, 100, 480, 20, "Passwort", true));
		this.accLabel = this.add(new Label(0, 150, 480, 20, "Zugang", true));
		this.shift();
	}
	
	public String getTitle() {
		return this.server == null ? "Mit Server verbinden" : (this.server.getName().isEmpty() ? "Server hinzufügen" : "Server bearbeiten");
	}
	
	private void connect() {
		if(this.gm.world != null)
			return;
		String name = null;
		if(this.server != null) {
			name = this.nameBox.getText();
			if(name.isEmpty()) {
				this.nameLabel.setText(TextColor.RED + "Name");
				return;
			}
		}
		String addr = this.addrBox.getText();
		if(addr.isEmpty()) {
			this.addrLabel.setText(TextColor.RED + "Adresse");
			return;
		}
		int port = -1;
		if(this.portBox.getText().isEmpty()) {
			this.portLabel.setText(TextColor.RED + "Port");
			return;
		}
		else {
			try {
				port = Integer.parseInt(this.portBox.getText());
			}
			catch(NumberFormatException e) {
			}
			if(port < 0 || port > 65535) {
				this.portLabel.setText(TextColor.RED + "Port");
				return;
			}
		}
		String user = this.userBox.getText();
		if(user.isEmpty()) {
			this.userLabel.setText(TextColor.RED + "Nutzer");
			return;
		}
		String pass = this.passBox.getText();
		String acc = this.accBox.getText();
		if(this.server == null) {
			this.lastAddr = addr;
			this.lastPort = port;
			this.lastUser = user;
			this.lastPass = pass;
			this.lastAcc = acc;
			this.gm.setDirty();
			this.gm.connect(addr, port, user, pass, acc);
		}
		else {
			this.server.setData(name, addr, port, user, pass, acc);
			GuiConnect.INSTANCE.applyServer(this.server);
		}
	}
	
	public void use(Textbox elem, Action value) {
		if(value == Action.SEND) {
			elem.setDeselected();
			this.connect();
		}
		else if(value == Action.FOCUS) {
			if(elem == this.addrBox)
				this.addrLabel.setText("Adresse");
			else if(elem == this.portBox)
				this.portLabel.setText("Port");
			else if(elem == this.userBox)
				this.userLabel.setText("Nutzer");
			else if(this.server != null && elem == this.nameBox)
				this.nameLabel.setText("Name");
		}
	}
}
