package client.gui.options;

import client.gui.Style;
import client.gui.element.ActButton;
import client.gui.element.ActButton.Mode;
import client.gui.element.Dropdown;
import client.gui.element.Element;
import client.gui.element.SelectableButton;
import client.gui.element.Slider;
import client.gui.element.Switch;
import client.gui.element.Textbox;
import client.gui.element.Textbox.Action;
import client.vars.CVar;
import client.vars.ColorVar;
import client.gui.element.Toggle;

public class GuiStyle extends GuiOptions implements Dropdown.Callback<String>, ActButton.Callback, Toggle.Callback, Switch.Callback<String>, Slider.Callback, Textbox.Callback {
	private static final String[] STYLE_CVARS = {
		"color_button_top",
		"color_textbox_top",
		"color_border_top",
		
		"color_button_btm",
		"color_textbox_btm",
		"color_border_btm",

		"color_button_text",
		"color_textbox_text",
		"color_label_text"
	};
	
	protected GuiStyle() {
	}
	
	protected Element addSelector(String cvar, int x, int y, int w, int h) {
		CVar cv = this.gm.getVar(cvar);
		if(cv instanceof ColorVar) {
			ColorVar color = (ColorVar)cv;
			if(!color.getDisplay().isEmpty())
				this.add(color.label(x, y - 20, w, 20));
			if(this.gm.style != Style.CUSTOM)
				return this.add(new Textbox(x, y, w, h, true, color.getFieldValue(this.gm.style)));
			else
				return this.add(color.editor(x, y, w, h));
		}
		return super.addSelector(cvar, x, y, w, h);
	}
	
	public void init(int width, int height) {
		int z;
		for(z = 0; z < STYLE_CVARS.length; z++) {
			this.addSelector(STYLE_CVARS[z], 10 + ((z % 3) + 1) * 190, 100 + z / 3 * 50, 180, 24);
		}
		z = 0;
		for(Style theme : Style.values()) {
			ActButton.Callback callback = new ActButton.Callback() {
				public void use(ActButton elem, Mode action) {
					if(GuiStyle.this.gm.style != theme) {
						GuiStyle.this.gm.style = theme;
						GuiStyle.this.gm.setDirty();
						GuiStyle.this.gm.displayGuiScreen(GuiStyle.this);
					}
				}
			};
			this.add( // theme == this.gm.style ? new SelectedButton(10 + (z % 3) * 320, 360 + (z / 3) * 40, 300, 24, callback, theme.name) : 
					new SelectableButton(10 + (z % 3) * 320, 360 + (z / 3) * 40, 300, 24, callback, theme.name, theme == this.gm.style));
			z++;
		}

		String[] values = new String[] {"VALUE 1", "VALUE 2"};
		this.add(new Dropdown(10, height - 74, 300, 24, false, values, values[1], values[0], this, "DROPDOWN"));
		this.add(new ActButton(330, height - 74, 300, 24, this, "BUTTON"));
		this.add(new Toggle(650, height - 74, 140, 24, false, true, this, "TOGGLE"));
		this.add(new Toggle(810, height - 74, 140, 24, true, false, this, "TOGGLE"));
		values = new String[] {"VALUE 1", "VALUE 2", "VALUE 3", "VALUE 4"};
		this.add(new Switch(10, height - 34, 300, 24, values, values[2], values[0], this, "ENUM"));
		this.add(new Slider(330, height - 34, 300, 24, 0, -20, 827, 60, 120, this, "SLIDER"));
		this.add(new Textbox(650, height - 34, 300, 24, 128, true, this, "FIELD"));
		
		if(this.gm.style != Style.CUSTOM) {
			this.add(new ActButton(200, 100 + 3 * 50, 560, 24, new ActButton.Callback() {
				public void use(ActButton elem, Mode action) {
					if(GuiStyle.this.gm.style != Style.CUSTOM) {
						GuiStyle.this.gm.style.copyToCustom();
						GuiStyle.this.gm.style = Style.CUSTOM;
						GuiStyle.this.gm.setDirty();
						GuiStyle.this.gm.displayGuiScreen(GuiStyle.this);
					}
				}
			}, "In angepasstes Design kopieren"));
		}
		else {
			this.add(new ActButton(200, 100 + 3 * 50, 560, 24, new ActButton.Callback() {
				public void use(ActButton elem, Mode action) {
					GuiStyle.this.gm.style = Style.CUSTOM;
					for(String cvar : STYLE_CVARS) {
						GuiStyle.this.gm.getVar(cvar).setDefault();
					}
					GuiStyle.this.gm.setDirty();
					GuiStyle.this.gm.displayGuiScreen(GuiStyle.this);
				}
			}, "Angepasstes Design zurücksetzen"));
		}
		super.init(width, height);
	}

	public String getTitle() {
		return "Benutzeroberfläche";
	}
	
	public void use(Textbox elem, Action value) {
	}
	public void use(Slider elem, int value) {
	}
	public void use(Switch<String> elem, String value) {
	}
	public void use(Toggle elem, boolean value) {
	}
	public void use(ActButton elem, Mode action) {
	}
	public void use(Dropdown<String> elem, String value) {
	}
}
