package common.dimension;

import java.util.Set;

import common.collect.Sets;

public final class Planet extends Dimension {
	private final Set<Moon> moons = Sets.newTreeSet();
	
	Planet(int id, String name) {
		super(id, name);
	}
	
	public Planet(int id, String name, int sky, int fog, int clouds, long orbit, long rotation, float gravity, float temperature, 
			int brightness) {
		this(id, name, sky, fog, clouds, orbit, rotation, 0.0f, gravity, temperature, brightness);
	}
	
	public Planet(int id, String name, int sky, int fog, int clouds, long orbit, long rotation, float offset, float gravity, 
			float temperature) {
		this(id, name, sky, fog, clouds, orbit, rotation, offset, gravity, temperature, 0);
	}
	
	public Planet(int id, String name, int sky, int fog, int clouds, long orbit, long rotation, float gravity, float temperature) {
		this(id, name, sky, fog, clouds, orbit, rotation, 0.0f, gravity, temperature, 0);
	}
	
	public Planet(int id, String name, int sky, int fog, int clouds, long orbit, long rotation, float offset, float gravity, 
			float temperature, int brightness) {
		super(id, name);
		this.setPhysics(orbit, rotation, offset, gravity, temperature, brightness);
		this.setTimeQualifier(7);
		this.setStarBrightness(0.5f).setDeepStarBrightness(0.0f);
		this.setSkyColor(sky).setFogColor(fog).setCloudColor(clouds);
	}
	
	public void addMoon(Moon moon) {
		this.moons.add(moon);
	}
	
	public final DimType getType() {
		return DimType.PLANET;
	}
}
