package server.biome;

import common.biome.Biome;
import common.entity.Entity;
import common.entity.types.EntityLiving;
import common.init.Blocks;
import common.init.EntityRegistry;
import common.rng.Random;
import common.rng.WeightedList;
import common.util.BlockPos;
import server.world.WorldServer;
import server.worldgen.FeatureGenerator;
import server.worldgen.foliage.WorldGenMushroom;

public class BiomeChaos extends GenBiome
{
    protected FeatureGenerator mushroomBlueGen = new WorldGenMushroom(Blocks.blue_mushroom);
    
    public BiomeChaos()
    {
        super(Biome.CHAOS);
        this.topBlock = Blocks.obsidian.getState();
        this.fillerBlock = Blocks.obsidian.getState();
    }
    
    protected void addMobs(WeightedList<RngSpawn> mobs) {
    	for(Class<? extends Entity> clazz : EntityRegistry.getAllClasses()) {
        	if(EntityLiving.class.isAssignableFrom(clazz))
        		mobs.add(new RngSpawn((Class<? extends EntityLiving>)clazz, 1, 1, 8));
        }
    }

    public void decorate(WorldServer worldIn, Random rand, BlockPos pos)
    {
    	super.decorate(worldIn, rand, pos);
    	
    	if (rand.chance(10))
        {
            int i8 = rand.chOffset();
            int l11 = rand.chOffset();
            BlockPos blockpos2 = worldIn.getHeight(pos.add(i8, 0, l11));
            this.mushroomBlueGen.generate(worldIn, rand, blockpos2);
        }
    }
}
