package server.biome;

import common.biome.Biome;
import common.block.BlockDirt;
import common.block.BlockDoublePlant;
import common.entity.animal.EntityHorse;
import common.init.Blocks;
import common.rng.Random;
import common.util.BlockPos;
import server.world.WorldServer;
import server.worldgen.ChunkPrimer;
import server.worldgen.tree.WorldGenSavanna;
import server.worldgen.tree.WorldGenTree;

public class BiomeSavanna extends GenBiome
{
    private static final WorldGenSavanna field_150627_aC = new WorldGenSavanna(false);

    protected BiomeSavanna(boolean plateau)
    {
        super(plateau ? Biome.SAVANNAPLATEAU : Biome.SAVANNA);
        this.mobs.add(new RngSpawn(EntityHorse.class, 1, 2, 6));
        this.treesPerChunk = 1;
        this.flowersPerChunk = 4;
        this.grassPerChunk = 20;
    }

    public WorldGenTree genBigTreeChance(Random rand)
    {
        return (WorldGenTree)(rand.rarity(5) ? field_150627_aC : this.worldGeneratorTrees);
    }

    protected GenBiome createMutatedBiome(Biome base)
    {
        GenBiome biomegenbase = new BiomeSavanna.Mutated(base, this);
        biomegenbase.depth = this.depth * 0.5F + 0.3F;
        biomegenbase.scale = this.scale * 0.5F + 1.2F;
        return biomegenbase;
    }

    public void decorate(WorldServer worldIn, Random rand, BlockPos pos)
    {
        DOUBLE_PLANT_GEN.setPlantType(BlockDoublePlant.EnumPlantType.GRASS);

        for (int i = 0; i < 7; ++i)
        {
            int j = rand.chOffset();
            int k = rand.chOffset();
            int l = rand.zrange(worldIn.getHeight(pos.add(j, 0, k)).getY() + 32);
            DOUBLE_PLANT_GEN.generate(worldIn, rand, pos.add(j, l, k));
        }

        super.decorate(worldIn, rand, pos);
    }

    public static class Mutated extends BiomeMutated
    {
        public Mutated(Biome base, GenBiome p_i45382_2_)
        {
            super(base, p_i45382_2_);
            this.treesPerChunk = 2;
            this.flowersPerChunk = 2;
            this.grassPerChunk = 5;
        }

        public void genTerrainBlocks(WorldServer worldIn, Random rand, ChunkPrimer chunkPrimerIn, int x, int z, double noiseVal)
        {
            this.topBlock = Blocks.grass.getState();
            this.fillerBlock = Blocks.dirt.getState();

            if (noiseVal > 1.75D)
            {
                this.topBlock = Blocks.stone.getState();
                this.fillerBlock = Blocks.stone.getState();
            }
            else if (noiseVal > -0.5D)
            {
                this.topBlock = Blocks.dirt.getState().withProperty(BlockDirt.VARIANT, BlockDirt.DirtType.COARSE_DIRT);
            }

            this.generateBiomeTerrain(worldIn, rand, chunkPrimerIn, x, z, noiseVal);
        }

        public void decorate(WorldServer worldIn, Random rand, BlockPos pos)
        {
            this.decorateNormal(worldIn, rand, pos);
        }
    }
}
