package server.biome;

import common.biome.Biome;
import common.init.Blocks;
import common.rng.Random;
import common.rng.WeightedList;
import common.util.BlockPos;
import server.world.WorldServer;
import server.worldgen.feature.WorldGenIcePath;
import server.worldgen.feature.WorldGenIceSpike;
import server.worldgen.tree.WorldGenTaiga2;
import server.worldgen.tree.WorldGenTree;

public class BiomeSnow extends GenBiome
{
    private final WorldGenIceSpike spikeGen = new WorldGenIceSpike();
    private final WorldGenIcePath pathGen = new WorldGenIcePath(4);
    private final boolean spiky;

    public BiomeSnow(Biome base, boolean spiky)
    {
        super(base);
        this.spiky = spiky;
        if(spiky)
            this.topBlock = Blocks.snow.getState();
    }
    
    protected void addMobs(WeightedList<RngSpawn> mobs) {
    }

    public void decorate(WorldServer worldIn, Random rand, BlockPos pos)
    {
        if (this.spiky)
        {
            for (int i = 0; i < 3; ++i)
            {
                int j = rand.chOffset();
                int k = rand.chOffset();
                this.spikeGen.generate(worldIn, rand, worldIn.getHeight(pos.add(j, 0, k)));
            }

            for (int l = 0; l < 2; ++l)
            {
                int i1 = rand.chOffset();
                int j1 = rand.chOffset();
                this.pathGen.generate(worldIn, rand, worldIn.getHeight(pos.add(i1, 0, j1)));
            }
        }

        super.decorate(worldIn, rand, pos);
    }

    public WorldGenTree genBigTreeChance(Random rand)
    {
        return new WorldGenTaiga2(false);
    }

    protected GenBiome createMutatedBiome(Biome base)
    {
        GenBiome biomegenbase = (new BiomeSnow(base, true)).enableColdBeach().setScaling(this.depth + 0.1F, this.scale + 0.1F);
        biomegenbase.depth = this.depth + 0.3F;
        biomegenbase.scale = this.scale + 0.4F;
        return biomegenbase;
    }
}
