package server.command;

import java.util.Collection;

import common.util.CharValidator;

public class StringParser extends DefaultingParser {
	private final boolean allowEmpty;
	private final Integer minLength;
	private final Integer maxLength;
	private final CharValidator validator;
	private final StringCompleter completer;
	
	public StringParser(String name, String def, boolean allowEmpty, Integer minLength, Integer maxLength, CharValidator validator, 
			Object ... completions) {
		super(name, def, completions);
		this.allowEmpty = allowEmpty;
		this.minLength = minLength;
		this.maxLength = maxLength;
		this.validator = validator;
		this.completer = null;
	}
	
	public StringParser(String name, String def, boolean allowEmpty, Integer minLength, Integer maxLength, CharValidator validator, 
			StringCompleter completer) {
		super(name, def);
		this.allowEmpty = allowEmpty;
		this.minLength = minLength;
		this.maxLength = maxLength;
		this.validator = validator;
		this.completer = completer;
	}
	
	public String parse(CommandEnvironment env, String input) {
		if(!this.allowEmpty && input.isEmpty())
			throw new RunException("Die Zeichenkette darf nicht leer sein");
		if(this.minLength != null && input.length() < this.minLength)
			if(this.maxLength != null)
				throw new RunException("Die Zeichenkette muss zwischen %d .. %d Zeichen lang sein, habe %d ('%s')", 
						this.minLength, this.maxLength, input.length(), input);
			else
				throw new RunException("Die Zeichenkette muss mindestens %d Zeichen lang sein, habe %d ('%s')", 
						this.minLength, input.length(), input);
		if(this.maxLength != null && input.length() > this.maxLength)
			if(this.minLength != null)
				throw new RunException("Die Zeichenkette muss zwischen %d .. %d Zeichen lang sein, habe %d ('%s')", 
						this.minLength, this.maxLength, input.length(), input);
			else
				throw new RunException("Die Zeichenkette darf höchstens %d Zeichen lang sein, habe %d ('%s')", 
						this.maxLength, input.length(), input);
		if(this.validator != null && !this.validator.valid(input))
			throw new RunException("Die Zeichenkette '%s' enthält ungültige Zeichen", input);
		return input;
	}

	public Class<?> getTypeClass(boolean required) {
		return String.class;
	}
	
	public Collection<String> getCompletions(CommandEnvironment env) {
		return this.completer != null ? this.completer.complete(env) : super.getCompletions(env);
	}
}
