package server.command.commands;

import common.color.TextColor;
import common.init.Config;
import common.network.IPlayer;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.RunException;
import server.network.Player;
import server.util.Form;

public class CommandPasswd extends Command {
	public CommandPasswd() {
		super("passwd");

		this.addPlayer("player", true);
		this.setParamsOptional();
		this.addString("password", false);
	}
	
	public void exec(CommandEnvironment env, Executor exec, Player player, String password) {
		if(exec.isPlayer()) {
			if(password != null)
				throw new RunException("Bei Verwendung als Spieler darf kein Passwort angegeben werden");
			if(player.getAdmin() && player != exec)
				throw new RunException("%s ist ein Admin", player.getUser());
			((Player)exec).displayForm(new Form() {
				private Field checkField;
				private Field passwordField;
				private Field confirmField;
				
				protected void init() {
					this.checkField = player != exec ? null : this.addPassword("Aktuelles Passwort", 0, IPlayer.MAX_PASS_LENGTH, "");
					this.passwordField = this.addPassword("Neues Passwort", Config.minPassLength, IPlayer.MAX_PASS_LENGTH, "");
					this.confirmField = this.addPassword("Passwort bestätigen", Config.minPassLength, IPlayer.MAX_PASS_LENGTH, "");
				}
				
				public String getTitle() {
					return "Passwort für " + player.getUser() + " ändern";
				}
				
				protected void accept() {
					Player plr = env.getServer().getPlayer(player.getUser());
					if(!((Player)exec).isAdmin() || plr == null || (plr.isAdmin() && plr != exec)) {
						exec.logConsole(TextColor.DRED + "Ein Fehler ist aufgetreten");
						return;
					}
					if(this.checkField != null && !this.checkField.get().equals(plr.getPassword())) {
						exec.logConsole(TextColor.RED + "Falsches Passwort eingegeben");
						return;
					}
					if(!this.passwordField.get().equals(this.confirmField.get())) {
						exec.logConsole(TextColor.RED + "Passwörter stimmen nicht überein");
						return;
					}
					plr.setPassword(this.passwordField.get());
					exec.logConsole(TextColor.GREEN + "Passwort" + (plr != exec ? " für %s" : "") + " gesetzt", plr.getUser());
				}
			});
		}
		else if(exec.isConsole()) {
			if(password == null)
				throw new RunException("Bei Verwendung in der Konsole muss ein Passwort angegeben werden");
			player.setPassword(password);
			exec.logConsole(TextColor.GREEN + "Passwort für %s gesetzt", player.getUser());
		}
	}
}
