package server.worldgen;

import common.rng.NoiseGen;
import common.rng.OctaveGen;
import common.rng.Random;
import common.world.State;
import server.world.WorldServer;

public class GeneratorSimple implements ChunkGenerator
{
	private final BiomeGenPerlin biomeGen;
    private final NoiseGen lowerNoiseGen;
    private final NoiseGen upperNoiseGen;
    private final NoiseGen mainNoiseGen;
    private final NoiseGen biomeNoiseGen;
    private final NoiseGen depthNoiseGen;

    private final State filler;
    private final State liquid;
    private final double noiseTable[] = new double[425];
    private final double mainNoise[] = new double[425];
    private final double lowerNoise[] = new double[425];
    private final double upperNoise[] = new double[425];
    private final double biomeNoise[] = new double[25];
    private final double depthNoise[] = new double[25];
    private final double[] factors = new double[256];
    
//    public GeneratorSimple(long seed, Random rand, GeneratorSettings settings)
//    {
//    	this(rand, settings.farlands, Blocks.stone.getDefaultState(), 
//    			settings.useLavaSeas ? Blocks.lava.getDefaultState() : Blocks.water.getDefaultState(),
//    			settings.biomeMode == 2 ? new BiomeGenPerlin(seed) : null);
//    }
    
//    public GeneratorSimple(Dimension dim, Random rand)
//    {
//    	this(rand, /* dim.hasFarLands(), */ dim.getFiller(), dim.getLiquid(), 
//    			dim.hasBiomes() ? null : new BiomeGenPerlin(rand.longv()));
//    }

    public GeneratorSimple(Random rand, /* boolean farlands, */ State filler, State liquid, BiomeGenPerlin biomeGen)
    {
    	this.lowerNoiseGen = /* farlands ? new OctaveGenOld(rand, 16) : */ new OctaveGen(rand, 16);
    	this.upperNoiseGen = /* farlands ? new OctaveGenOld(rand, 16) : */ new OctaveGen(rand, 16);
    	this.mainNoiseGen = /* farlands ? new OctaveGenOld(rand, 8) : */ new OctaveGen(rand, 8);
    	this.biomeNoiseGen = /* farlands ? new OctaveGenOld(rand, 10) : */ new OctaveGen(rand, 10);
    	this.depthNoiseGen = /* farlands ? new OctaveGenOld(rand, 16) : */ new OctaveGen(rand, 16);
        this.filler = filler;
        this.liquid = liquid;
        this.biomeGen = biomeGen;
    }
    
	public int getMaximumHeight() {
		return 128;
	}

    public void generateChunk(WorldServer world, int x, int z, ChunkPrimer primer)
    {
    	if(this.biomeGen == null)
    		world.getBiomeGenerator().genFactors(this.factors, x * 16, z * 16, 16, 16);
    	else
    		this.biomeGen.generate(this.factors, x, z);
        byte range = 4;
        int sea = world.getSeaLevel();
        int xr = range + 1;
        byte yr = 17;
        int zr = range + 1;
        this.genNoisemap(x * range, 0, z * range);
        for(int xb = 0; xb < range; xb++)
        {
            for(int zb = 0; zb < range; zb++)
            {
                for(int yb = 0; yb < 16; yb++)
                {
                    double mul1 = 0.125D;
                    double d1 = this.noiseTable[((xb + 0) * zr + (zb + 0)) * yr + (yb + 0)];
                    double d2 = this.noiseTable[((xb + 0) * zr + (zb + 1)) * yr + (yb + 0)];
                    double d3 = this.noiseTable[((xb + 1) * zr + (zb + 0)) * yr + (yb + 0)];
                    double d4 = this.noiseTable[((xb + 1) * zr + (zb + 1)) * yr + (yb + 0)];
                    double d5 = (this.noiseTable[((xb + 0) * zr + (zb + 0)) * yr + (yb + 1)] - d1) * mul1;
                    double d6 = (this.noiseTable[((xb + 0) * zr + (zb + 1)) * yr + (yb + 1)] - d2) * mul1;
                    double d7 = (this.noiseTable[((xb + 1) * zr + (zb + 0)) * yr + (yb + 1)] - d3) * mul1;
                    double d8 = (this.noiseTable[((xb + 1) * zr + (zb + 1)) * yr + (yb + 1)] - d4) * mul1;
                    for(int ys = 0; ys < 8; ys++)
                    {
                        double mul2 = 0.25D;
                        double n1 = d1;
                        double n2 = d2;
                        double n3 = (d3 - d1) * mul2;
                        double n4 = (d4 - d2) * mul2;
                        for(int xs = 0; xs < 4; xs++)
                        {
//                            int off = xs + xb * 4 << 11 | 0 + zb * 4 << 7 | yb * 8 + ys;
//                            char zo = '\200';
                            double mul3 = 0.25D;
                            double val = n1;
                            double step = (n2 - n1) * mul3;
                            for(int zs = 0; zs < 4; zs++)
                            {
                                if(val > 0.0D)
                                {
                                	primer.set(xs + xb * 4, ys + yb * 8, zs + zb * 4, this.filler);
//                                	primer.setBlockStateLegacy(off, this.filler);
                                }
                                else if(yb * 8 + ys < sea)
                                {
                                	primer.set(xs + xb * 4, ys + yb * 8, zs + zb * 4, this.liquid);
//                                	primer.setBlockStateLegacy(off, this.liquid);
                                }
//                                off += zo;
                                val += step;
                            }

                            n1 += n3;
                            n2 += n4;
                        }

                        d1 += d5;
                        d2 += d6;
                        d3 += d7;
                        d4 += d8;
                    }

                }

            }

        }

    }

    private void genNoisemap(int x, int y, int z)
    {
    	int xs = 5;
    	int ys = 17;
    	int zs = 5;
        double scale = 684.41200000000003D;
        double height = 684.41200000000003D;
//        double temps[] = this.biomeGen.temperature;
//        double rains[] = this.biomeGen.humidity;
        this.biomeNoiseGen.generate(this.biomeNoise, x, z, xs, zs, 1.121D, 1.121D, 0.5D);
        this.depthNoiseGen.generate(this.depthNoise, x, z, xs, zs, 200D, 200D, 0.5D);
        this.mainNoiseGen.generate(this.mainNoise, x, y, z, xs, ys, zs, scale / 80D, height / 160D, scale / 80D);
        this.lowerNoiseGen.generate(this.lowerNoise, x, y, z, xs, ys, zs, scale, height, scale);
        this.upperNoiseGen.generate(this.upperNoise, x, y, z, xs, ys, zs, scale, height, scale);
        int pos = 0;
        int dpos = 0;
        int ps = 16 / xs;
        for(int u = 0; u < xs; u++)
        {
            int px = u * ps + ps / 2;
            for(int v = 0; v < zs; v++)
            {
                int pz = v * ps + ps / 2;
//                double temp = temps[px * 16 + pz];
//                double rain = rains[px * 16 + pz] * temp;
                double fact = 1.0D - this.factors[pz * 16 + px];
                fact *= fact;
                fact *= fact;
                fact = 1.0D - fact;
                double high = (this.biomeNoise[dpos] + 256D) / 512D;
                high *= fact;
                if(high > 1.0D)
                {
                    high = 1.0D;
                }
                double depth = this.depthNoise[dpos] / 8000D;
                if(depth < 0.0D)
                {
                    depth = -depth * 0.29999999999999999D;
                }
                depth = depth * 3D - 2D;
                if(depth < 0.0D)
                {
                    depth /= 2D;
                    if(depth < -1D)
                    {
                        depth = -1D;
                    }
                    depth /= 1.3999999999999999D;
                    depth /= 2D;
                    high = 0.0D;
                } else
                {
                    if(depth > 1.0D)
                    {
                        depth = 1.0D;
                    }
                    depth /= 8D;
                }
                if(high < 0.0D)
                {
                    high = 0.0D;
                }
                high += 0.5D;
                depth = (depth * (double)ys) / 16D;
                double shift = (double)ys / 2D + depth * 4D;
                dpos++;
                for(int w = 0; w < ys; w++)
                {
                    double noise = 0.0D;
                    double damp = (((double)w - shift) * 12D) / high;
                    if(damp < 0.0D)
                    {
                        damp *= 4D;
                    }
                    double lower = this.lowerNoise[pos] / 512D;
                    double upper = this.upperNoise[pos] / 512D;
                    double base = (this.mainNoise[pos] / 10D + 1.0D) / 2D;
                    if(base < 0.0D)
                    {
                        noise = lower;
                    } else
                    if(base > 1.0D)
                    {
                        noise = upper;
                    } else
                    {
                        noise = lower + (upper - lower) * base;
                    }
                    noise -= damp;
                    if(w > ys - 4)
                    {
                        double d13 = (float)(w - (ys - 4)) / 3F;
                        noise = noise * (1.0D - d13) + -10D * d13;
                    }
                    this.noiseTable[pos] = noise;
                    pos++;
                }

            }

        }
    }
}
