package server.worldgen.feature;

import common.block.Block;
import common.init.Blocks;
import common.rng.Random;
import common.util.BlockPos;
import common.util.ExtMath;
import server.world.WorldServer;

public class WorldGenClayExt extends WorldGenClay {
	public WorldGenClayExt(int blocks) {
		super(blocks);
	}

	public boolean generate(WorldServer worldIn, Random rand, BlockPos position) {
		if(!worldIn.getState(position).getBlock().getMaterial().isColdLiquid()) {
			return false;
		}
		float f = rand.floatv() * 3.141593F;
		double d = (float)position.getX() + (ExtMath.sin(f) * (float)numberOfBlocks) / 8F;
		double d1 = (float)position.getX() - (ExtMath.sin(f) * (float)numberOfBlocks) / 8F;
		double d2 = (float)position.getZ() + (ExtMath.cos(f) * (float)numberOfBlocks) / 8F;
		double d3 = (float)position.getZ() - (ExtMath.cos(f) * (float)numberOfBlocks) / 8F;
		double d4 = position.getY() + rand.zrange(3) + 2;
		double d5 = position.getY() + rand.zrange(3) + 2;
		for(int l = 0; l <= numberOfBlocks; l++) {
			double d6 = d + ((d1 - d) * (double)l) / (double)numberOfBlocks;
			double d7 = d4 + ((d5 - d4) * (double)l) / (double)numberOfBlocks;
			double d8 = d2 + ((d3 - d2) * (double)l) / (double)numberOfBlocks;
			double d9 = (rand.doublev() * (double)numberOfBlocks) / 16D;
			double d10 = (double)(ExtMath.sin(((float)l * 3.141593F) / (float)numberOfBlocks) + 1.0F) * d9 + 1.0D;
			double d11 = (double)(ExtMath.sin(((float)l * 3.141593F) / (float)numberOfBlocks) + 1.0F) * d9 + 1.0D;
			for(int i1 = (int)(d6 - d10 / 2D); i1 <= (int)(d6 + d10 / 2D); i1++) {
				for(int j1 = (int)(d7 - d11 / 2D); j1 <= (int)(d7 + d11 / 2D); j1++) {
					for(int k1 = (int)(d8 - d10 / 2D); k1 <= (int)(d8 + d10 / 2D); k1++) {
						double d12 = (((double)i1 + 0.5D) - d6) / (d10 / 2D);
						double d13 = (((double)j1 + 0.5D) - d7) / (d11 / 2D);
						double d14 = (((double)k1 + 0.5D) - d8) / (d10 / 2D);
						if(d12 * d12 + d13 * d13 + d14 * d14 >= 1.0D) {
							continue;
						}
						BlockPos blockpos = new BlockPos(i1, j1, k1);
						Block block = worldIn.getState(blockpos).getBlock();

						if(block == Blocks.sand) {
							worldIn.setState(blockpos, this.clayBlock.getState(), 2);
						}
					}
				}
			}
		}
		return true;
	}
}
