package server.worldgen.structure;

import java.util.Arrays;
import java.util.List;

import common.biome.Biome;
import common.rng.Random;
import common.util.BlockPos;
import server.world.WorldServer;

public class MapGenScatteredFeature extends MapGenStructure
{
    private static final List<Biome> biomelist = Arrays.<Biome>asList(Biome.DESERT, Biome.DESERTHILLS, Biome.JUNGLE, Biome.JUNGLEHILLS, Biome.SWAMPLAND);
    private static final int MAX_DISTANCE = 32;
    private static final int MIN_DISTANCE = 8;

    public String getStructureName()
    {
        return "Temple";
    }

    protected boolean canSpawnStructureAtCoords(int chunkX, int chunkZ)
    {
        int i = chunkX;
        int j = chunkZ;

        if (chunkX < 0)
        {
            chunkX -= MapGenScatteredFeature.MAX_DISTANCE - 1;
        }

        if (chunkZ < 0)
        {
            chunkZ -= MapGenScatteredFeature.MAX_DISTANCE - 1;
        }

        int k = chunkX / MapGenScatteredFeature.MAX_DISTANCE;
        int l = chunkZ / MapGenScatteredFeature.MAX_DISTANCE;
        Random random = this.worldObj.getRng(k, l, 14357617);
        k = k * MapGenScatteredFeature.MAX_DISTANCE;
        l = l * MapGenScatteredFeature.MAX_DISTANCE;
        k = k + random.zrange(MapGenScatteredFeature.MAX_DISTANCE - MapGenScatteredFeature.MIN_DISTANCE);
        l = l + random.zrange(MapGenScatteredFeature.MAX_DISTANCE - MapGenScatteredFeature.MIN_DISTANCE);

        if (i == k && j == l)
        {
        	Biome biomegenbase = this.worldObj.getBiomeGenerator().getBiomeGenerator(new BlockPos(i * 16 + 8, 0, j * 16 + 8), null);

            if (biomegenbase == null)
            {
                return false;
            }

            for (Biome biomegenbase1 : biomelist)
            {
                if (biomegenbase == biomegenbase1)
                {
                    return true;
                }
            }
        }

        return false;
    }

    protected StructureStart getStructureStart(int chunkX, int chunkZ)
    {
        return new MapGenScatteredFeature.Start(this.worldObj, this.rand, chunkX, chunkZ);
    }

    public boolean hasMageHut(BlockPos pos)
    {
        StructureStart structurestart = this.func_175797_c(pos);

        if (structurestart != null && structurestart instanceof MapGenScatteredFeature.Start && !structurestart.components.isEmpty())
        {
            StructureComponent structurecomponent = (StructureComponent)structurestart.components.getFirst();
            return structurecomponent instanceof StructureScattered.SwampHut;
        }
        else
        {
            return false;
        }
    }

    public static class Start extends StructureStart
    {
        public Start()
        {
        }

        public Start(WorldServer worldIn, Random p_i2060_2_, int p_i2060_3_, int p_i2060_4_)
        {
            super(p_i2060_3_, p_i2060_4_);
            Biome biomegenbase = worldIn.getBiomeGenForCoords(new BlockPos(p_i2060_3_ * 16 + 8, 0, p_i2060_4_ * 16 + 8));

            if (biomegenbase != Biome.JUNGLE && biomegenbase != Biome.JUNGLEHILLS)
            {
                if (biomegenbase == Biome.SWAMPLAND)
                {
                    StructureScattered.SwampHut componentscatteredfeaturepieces$swamphut = new StructureScattered.SwampHut(p_i2060_2_, p_i2060_3_ * 16, p_i2060_4_ * 16);
                    this.components.add(componentscatteredfeaturepieces$swamphut);
                }
                else if (biomegenbase == Biome.DESERT || biomegenbase == Biome.DESERTHILLS)
                {
                    StructureScattered.DesertPyramid componentscatteredfeaturepieces$desertpyramid = new StructureScattered.DesertPyramid(p_i2060_2_, p_i2060_3_ * 16, p_i2060_4_ * 16);
                    this.components.add(componentscatteredfeaturepieces$desertpyramid);
                }
            }
            else
            {
                StructureScattered.JunglePyramid componentscatteredfeaturepieces$junglepyramid = new StructureScattered.JunglePyramid(p_i2060_2_, p_i2060_3_ * 16, p_i2060_4_ * 16);
                this.components.add(componentscatteredfeaturepieces$junglepyramid);
            }

            this.updateBoundingBox();
        }
    }
}
